%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%                                                                   %%%
%%%	MATLAB Code for:					      									 %%% 
%%%								      												 %%%
%%%			GENERATIONAL ACCOUNTING			      							 %%%
%%%								      												 %%%
%%%								      												 %%%
%%%	Programming by:  	Philip Oreopoulos		      						 %%%
%%%								      												 %%%
%%%	Filename:		gamatnew.m			      								 %%%
%%%								      												 %%%
%%%								      												 %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;

%==================================%
%  Parameter Settings and Switches %
%==================================%

% policy adjustment parameters:

switch04 = 0;				% policy adjustment for all gen. if 1;
startingyear = 1995;		% starting year of policy if currentgen=1
								% note: this parameter is not set
                        % at the moment, the policy change starts
                        % in the base year - this can easily be changed
                        
numtaxtypes = 5;			% total number of tax types to adjust
numtransfertypes = 8;	% total number of transfer types to adjust

taxcat = [9 10 11 14 15];		
transfercat = [1 2 3 4 5 6 7 8];
								% matrix of tax/transfer category numbers

% Codes are:
%						Taxes								Tramsfers

%					9  Labour Income Taxes			1  OASDI
%					10  FICA Taxes						2  Medicare		
%					11 Excise Taxes					3  Medicaid 
%					12 Cap. Income Taxes				4  UI
%					13 Inframarginal Cap. 			5  General Welfare
%					14 Monetary Base					6  AFDC
%					15 Property Taxes					7 Food Stamps
%															8 education
%															15 Fed. Government Cons.
%															16 State Government Cons.



DEBT = 2078610000000;

switch01 = 0;				% omits cap. tax. adj. if '1'
switch02 = 1;				% also calculates m & f together
switch05 = 0;				% equalize burdens experiment
switch06 = 2;				% specifies aggregate projections
								% 1=low,2=med,3=high,4=with edu.
switch07 = 2;				% specifies demographic projections
								% (1=low, 2=med, 3=high)
switch08 = 1;				% seignorage adjustment                        

r=0.060;						
g=0.012;						% note: adjust growth rates below
ages=101;     				
types=17;    				% note: types 15, 16 are govcon

			
baseyear 	= 1995;			
lastyear 	= 2100;			
projyear 	= 2070;			

											% these definitions make the notation
prjyear=projyear-baseyear+1;		% easier to follow when working
lstyear=lastyear-baseyear+1;		% with column and row numbers
remainyear=lastyear-projyear+1;



%=====================================%
% Load in data files and conform data %
%=====================================%

if switch07 == 1
   load POPMLOW.txt;POPM=POPMLOW;clear POPMLOW;
   load POPFLOW.txt;POPF=POPFLOW;clear POPFLOW;
elseif switch07 == 2
   load POPMMED.txt;POPM=POPMMED;clear POPMMED;
   load POPFMED.txt;POPF=POPFMED;clear POPFMED;
elseif switch07 == 3
   load POPMHIGH.txt;POPM=POPMHIGH;clear POPMHIGH;
   load POPFHIGH.txt;POPF=POPFHIGH;clear POPFHIGH;
end


POPM(:,1)=[];		
POPF(:,1)=[];


load RPM.txt; 		% loads relative profile data	
load RPF.txt;		% indexed by TYPE x AGE
RPM(:,1:2)=[];		% 1st col type, 2nd col year are removed
RPF(:,1:2)=[];		


RPM(14,1)=0;RPF(14,1)=0;
for i=2:ages;
   RPM(14,i)=RPM(14,i)-RPM(14,i-1);
   RPF(14,i)=RPF(14,i)-RPF(14,i-1);
end;
rm40=RPM(14,40);
for i=1:ages;
   RPM(14,i)=RPM(14,i)/rm40;
   RPF(14,i)=RPF(14,i)/rm40;
end;



RP(:,:,:,1) = repmat(permute(RPM,[2,3,1,4]),[1,prjyear,1,1]);
RP(:,:,:,2) = repmat(permute(RPF,[2,3,1,4]),[1,prjyear,1,1]);

			% RP matrix is relative profiles with
			% dims RP(age,year,type,sex), and the
			% year goes from baseyear to projyear  

if switch06 == 1
   load AGGLO.txt; AGG=AGGLO;clear AGGLO;g=0.07;
elseif switch06 == 2
   load AGGMED.txt; AGG=AGGMED;clear AGGMED;g=0.012;
elseif switch06 == 3
   load AGGHI.txt; AGG=AGGHI;clear AGGHI;g=0.017;
elseif switch06 == 4
   load AGGEDUC.txt; AGG=AGGEDUC;clear AGGEDUC;g=0.012;
end


				      	% Aggregate matrix has dimensions
AGG(:,1)=[];			% YEAR x TYPE. first clumn
							% displaying the year is removed
							% NOTE: the last two columns contain
							% the values of federal and state government
							% consumption
 

%---------------------------------------------------------------%
% Gets discount and growth matricies with dims AGE x YEAR, from %
% baseyear to lastyear: (1+g)^0, (1+g)^1, (1+g)^2, ...          %
%---------------------------------------------------------------%

for i = 1:lstyear,DSCT(:,i) = (1/(1+r)^(i-1))*ones(ages,1);,end

for j = 1:lstyear,GRTH(:,j) = (1+g)^(j-1)*ones(ages,1);,end

%==================================================%
% Converts Aggregates to billions of U.S. dollars  %
% and defines some variables			   				%
%==================================================%

AGG = 1000000000*AGG;


%===============================================%
%																%
% Capital Income Tax Adjustment (If switch01=0) %
%																%			
%===============================================%

qcap 	= 0.11100;
delcap 	= 0.00111;

if switch01 == 1, AGG(:,13) = 0;, end;

for i = 1:prjyear;
   AGG(i,12) = AGG(i,12) - (delcap*AGG(1,13))*((1+g)^(i-1));
end;
AGG(:,13) = qcap*AGG(:,13);

%=======================================================%
%																		  %
% Calculates from data per capita amounts of taxes paid %
% and transfers received for each age group by year     %
% 																		  %
%        PERCAP matrix is (age,year,type,sex)			  %
%																		  %
%=======================================================%

SUMS = RPM*POPM(1:prjyear,:)' + RPF*POPF(1:prjyear,:)'; 

UNITIES = AGG./SUMS';

PERCAP(:,:,:,1) = repmat(permute(RPM,[2,3,1]),[1,prjyear,1]) .* ...
	   	    repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);
PERCAP(:,:,:,2) = repmat(permute(RPF,[2,3,1]),[1,prjyear,1]) .* ...
	            repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);

PERCAP(:,prjyear+1:ages+1,:,:) = ...
	repmat(PERCAP(:,prjyear,:,:),[1,ages-prjyear+1,1,1]) .* ...
	repmat(GRTH(:,2:ages-prjyear+2),[1,1,types,2]);

 
clear SUMS UNITIES;


%==================================================%
%						   										%
% converts values to present value, and calculates %
% generational accounts for living age groups      %
%						   										%
%==================================================%

POP(:,:,1,1) = POPM'; clear POPM;
POP(:,:,1,2) = POPF'; clear POPF;

PVTOTALS = PERCAP .* repmat(DSCT(:,1:ages+1),[1,1,types,2]) .* ...
   		     repmat(POP(:,1:ages+1,:,:),[1,1,types,1]);



for type = 1:types-2, for age = 1:ages

   TaxTransfers(age,type,1) = ...
	sum(diag(PVTOTALS(:,1:ages,type,1),-(age-1)))./POP(age,1,1,1);
   TaxTransfers(age,type,2) = ...
	sum(diag(PVTOTALS(:,1:ages,type,2),-(age-1)))./POP(age,1,1,2);

end, end

GABEFORE(:,1) = sum(TaxTransfers(:,:,1),2);	% sums along the rows in AVERAGES
GABEFORE(:,2) = sum(TaxTransfers(:,:,2),2);

GAFUTUREB = GABEFORE(1,:)*(1+g);

%==================================================%
%	                                 	   			%
% calculates males and females together if		   %
%						   										%
% switch02=1							   					%
%						   										%
%==================================================%

if switch02 == 1
   MFTOTALS=sum(PVTOTALS,4);
   MFPOP=sum(POP(:,1:2,1,:),4);
   
   for type = 1:types, for age = 1:ages    
		TAXTRANSBEFORE(age,type,3) = ...
 		sum(diag(MFTOTALS(:,1:ages,type,1),-(age-1))) ./ ...
													MFPOP(age,1);
	end, end

	GABEFORE(:,3) = sum(TAXTRANSBEFORE(:,1:15,3),2);	
end



%==============================================================%
%							       												%
% calculate gap if no policy change                            %
%							       												%
%==============================================================%

% The gap is the debt -
% the sum of all PV net payments for living generations -
% the PV net payments for future generations up to 2200 -
% the PV net payments for future generations extended out to infinity


netpvtaxes = sum(sum(sum(sum(PVTOTALS(:,1:ages,:,:))))) + 			...
								 														...
								 														...
	     sum(sum(sum(sum(                           	 				...
	repmat(PERCAP(:,ages+1,:,:,:),[1,lstyear-ages,1,1]) .* 	 		...
	repmat(GRTH(:,1:lstyear-ages),[1,1,types,2]) .* 	 				...
	repmat(DSCT(:,ages+1:lstyear),[1,1,types,2])   .* 	 				...
   repmat(POP(:,ages+1:lstyear,:,:,:),[1,1,types,1]))))) 			...
								 														...
				+				 														...
								 														...
	     sum(sum(sum(sum(                           	 				...
	PERCAP(:,ages+1,:,:) .* 	 			 									...
	repmat(GRTH(:,lstyear-ages),[1,1,types,2]) .* 	 					...
	repmat(DSCT(:,lstyear),[1,1,types,2])   .* 	         			...
        repmat(POP(:,lstyear,:,:,:),[1,1,types,1]))))) *      		...
	(1+g)*(1/(1-((1+g)/(1+r))));	       


govgap = DEBT - netpvtaxes;


%==============================================================%
%							       												%
% estimate percentage increase/decrease of tax/transfer types  %
% specified to eliminate gap.	                               	%
%							       												%
%	calculated for FUTURE generations only		       				%
%							       												%
%==============================================================%

% the percentage increase in particular tax/transfer programs is
% caclulated by summing all the PV payments/receipts of these
% programs for future generations, out to infinity, and calculating
% how much increase is required to remove the gap

% we calculate for tax and transfers seperately

% NOTE: the 'triu,1' command keeps the upper triangular portion of the 
% PERCAP matrix, shifted to the right one, which is the PV TOTALS for 
% future generations between 1994 and 2094.  The rest of the matrix 
% (for the percap amounts for living generations) become zero


for j=1:numtaxtypes; 

taxtotal(j) = 																				...
																								...	
sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),1),1)))) + 						...
sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),2),1)))) +						...						
								 																...
	     sum(sum(sum(sum(     	                      	 						...
repmat(PERCAP(:,ages+1,taxcat(j),:),[1,lstyear-ages,1,1]).* 				...
repmat(GRTH(:,1:lstyear-ages),[1,1,1,2]) .* 	 									...	
repmat(DSCT(:,ages+1:lstyear),[1,1,1,2])   .* 									...
repmat(POP(:,ages+1:lstyear,:,:,:),[1,1,1,1])))))	 							...
								 																...
				+				 																...
								 																...
	     sum(sum(sum(sum(                           	 						...
	PERCAP(:,ages+1,taxcat(j),:) .* 	 												...
	repmat(GRTH(:,lstyear-ages),[1,1,1,2]) .* 									...
	repmat(DSCT(:,lstyear),[1,1,1,2])   .* 	        							...
        repmat(POP(:,lstyear,:,:,:),[1,1,1,1]))))) *       					...
	(1+g)*(1/(1-((1+g)/(1+r))));	       

end


if numtaxtypes == 0, taxtotal=[0];,end;

for k=1:numtransfertypes; 

transfertotal(k) =  -																	...
																								...
sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),1),1)))) - 				...
sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),2),1)))) -   			...
																								...
								 																...
	     sum(sum(sum(sum(     	                      	 						...
repmat(PERCAP(:,ages+1,transfercat(k),:),[1,lstyear-ages,1,1]).*			...
repmat(GRTH(:,1:lstyear-ages),[1,1,1,2]) .* 	 									...	
repmat(DSCT(:,ages+1:lstyear),[1,1,1,2])   .* 									...
repmat(POP(:,ages+1:lstyear,:,:,:),[1,1,1,1]))))) 								...
								 																...
				-				 																...
								 																...
	     sum(sum(sum(sum(                           	 						...
	PERCAP(:,ages+1,transfercat(k),:) .* 	 										...
	repmat(GRTH(:,lstyear-ages),[1,1,1,2]) .* 									...
	repmat(DSCT(:,lstyear),[1,1,1,2])   .* 										...
        repmat(POP(:,lstyear,:,:,:),[1,1,1,1]))))) *  						...
	(1+g)*(1/(1-((1+g)/(1+r))));	       

end


if numtransfertypes == 0, transfertotal=[0];,end;



			% NOTE: we subtract to make the aggregate positive

percentagechange = govgap / ( sum(taxtotal) + sum(transfertotal) )



%==============================================================%
%							       												%
% policy adjustment for living generations if switch04=1       %
%							       												%
%==============================================================%

% simply adds on the additional PV net tax payments for living generations
% and then calculates the percentage change again

if switch04 ==1,
   
for j=1:numtaxtypes;
	taxtotal(j) = 	taxtotal(j) + ...
   sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),1))))) + ...
   sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),2)))));
end

if numtaxtypes == 0, taxtotal=[0];,end;

for k=1:numtransfertypes; 
	transfertotal(k) =  transfertotal(k) - ...
   sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),1))))) - ...
   sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),2)))));
end
 
if numtransfertypes == 0, transfertotal=[0];,end;


percentagechange = (govgap / (sum(taxtotal(:)) + sum(transfertotal(:))) )



end;



%==============================================================%
%							       												%
% computes new generational accounts, after policy change      %
% for future generations affected only...		       				%
%							       												%
%==============================================================%


if switch04==0,

   TAXTRANSAFTER=TAXTRANSBEFORE;
   
   GAAFTER = GABEFORE;


for j=1:numtaxtypes; for sex = 1:2;  
	PERCAP(:,1:ages+1,taxcat(j),sex) = ...
	tril(PERCAP(:,1:ages+1,taxcat(j),sex)) + ...
	(triu(PERCAP(:,1:ages+1,taxcat(j),sex),1)*(1+percentagechange));
end, end;

for k=1:numtransfertypes; for sex = 1:2;
	PERCAP(:,:,transfercat(k),sex) = ...
	tril(PERCAP(:,:,transfercat(k),sex)) + ...
	triu(PERCAP(:,:,transfercat(k),sex),1)*(1-percentagechange);
	end, end;

PVTOTALSF = PERCAP(:,2:102,:,:) .* ...
            repmat(DSCT(:,1:ages),[1,1,types,2]) .* ...
   		   repmat(POP(:,2:ages+1,:,:,:),[1,1,types,1]);

for type=1:types;, for sex = 1:2;
FutureTaxTrans(type,sex) = ...
	 sum(diag(PVTOTALSF(:,:,type,sex),0)) ./ ...
					POP(1,2,1,sex);
end,end



GAFUTUREA = sum(FutureTaxTrans(1:15,:))/(1+g);


%==============================================%
% combines for males and females if switch02=1 %
%==============================================%

if switch02 == 1
   MFTOTALS=sum(PVTOTALS,4);
   
   for type = 1:types    
		FutureTaxTrans(type,3) = ...
 		sum(diag(MFTOTALS(:,:,type,1),1)) ./ ...
													MFPOP(1,2);
	end

GAFUTUREA(:,3)=sum(FutureTaxTrans(1:15,3))/(1+g);
end


%================================================%
%						 										 %
% Computes new GAs for all, if switch04=1        %
%						 										 %
%================================================%

elseif switch04==1

      
for j=1:numtaxtypes;
PERCAP(:,:,taxcat(j),:) = PERCAP(:,:,taxcat(j),:)*(1+percentagechange);
end

for k=1:numtransfertypes;
PERCAP(:,:,transfercat(k),:) = ...
		PERCAP(:,:,transfercat(k),:) * (1-percentagechange);
end

PVTOTALS = PERCAP .* repmat(DSCT(:,1:ages+1),[1,1,types,2]) .* ...
   		     repmat(POP(:,1:ages+1,:,:),[1,1,types,1]);

for type = 1:types, for age = 1:ages, for sex = 1:2, 
   TAXTRANSAFTER(age,type,sex) = ...
	sum(diag(PVTOTALS(:,1:ages,type,sex),-(age-1))) ./ ...
						POP(age,1,1,sex);
end, end, end

for sex = 1:2
	GAAFTER(:,sex) = sum(TAXTRANSAFTER(:,1:15,sex),2);	
end

% same now for future generations (the cohort born after
% the base year for comparison purposes


PVTOTALSF = PERCAP(:,2:102,:,:) .* ...
            repmat(DSCT(:,1:ages),[1,1,types,2]) .* ...
   		   repmat(POP(:,2:ages+1,:,:),[1,1,types,1]);

for type=1:types;, for sex = 1:2;
FutureTaxTrans(type,sex) = ...
	 sum(diag(PVTOTALSF(:,:,type,sex),0)) ./ ...
					POP(1,2,1,sex);
end,end,end



GAFUTUREA = sum(FutureTaxTrans(1:15,:))/(1+g);


%GAFUTUREA = GAAFTER(1,:);





%================================================================%
% calculates GAs with males and females together if switch02 = 1 %
%================================================================%

if switch02 == 1
   MFTOTALS=sum(PVTOTALS,4);
   
   for type = 1:types, for age = 1:ages;    
		TAXTRANSAFTER(age,type,3) = ...
 		sum(diag(MFTOTALS(:,1:ages,type,1),-(age-1))) ./ ...
													MFPOP(age,1,1,1);
	end, end

	GAAFTER(:,3) = sum(TAXTRANSBEFORE(:,1:15,3),2);	
	GAFUTUREA(1,3) = GAAFTER(1,3)/(1+g);
end

end

%===========================================%
% prints males GAs by 5 year age categories %
%===========================================%

format bank
GAAFTER(1:5:91,1)/1000
GAFUTUREA(1,1)/1000




